$(document).ready(function(){

   /* example of data
   data["programid"]    = "tennis scheduler";
   data["persons"]      = {};
   data["players"]      = [];
   data["offdays"]      = {};
   data["startdate"]    = "";
   data["ndates"]       = 0;
   data["dayplayers"]   = 4;
   data["daypayers"]    = 1;
   data["fiddle"]       = 1;
   data["varia"]        = "";

   // example:

   data["persons"]["beckenbauer"]           = {};
   data["persons"]["beckenbauer"]["absent"] = {'2012-02-04':1,'2012-02-06':1};
   data["persons"]["beckenbauer"]["email"]  = 'fred@example.com'; 
   data["persons"]["beckenbauer"]["phone"]  = '06-999900'; 
   data["persons"]["beckenbauer"]["name"]   = 'Hans B.'; 
   */

   const weekdays = ['zondag','maandag','dinsdag','woensdag','donderdag','vrijdag','zaterdag'];

   const programid = "Willem's tennis scheduler";

   let data = {}; 
   let table;
   let filename    = "";
   let filecontent = "";

   function date2iso(d)
   {
      var t = d.toISOString();
      return t.split("T")[0];
   }

   function nextdate(d) {
      let nd = new Date(d+'T12:00:00');
      nd.setDate(nd.getDate() + 7);  // increment by 7 days
      return date2iso(nd);
   }

   function initpicker() {

      $('#firstdate').simplePicker({
	 // add style to datepicker
	 style: 'white',
	 // set first day in a week 0 = Sunday, 1 = Monday ...
	 firstday: 1,
	 // declare names of the days
	 days: ['Zo', 'Ma', 'Di', 'Wo', 'Do', 'Vr', 'Za'],
	 // declare names of the months
	 months: ['Januari', 'Februari', 'Maart', 'April', 'Mei', 'Juni', 'Juli', 'Augustus', 'September', 'Oktober', 'November', 'December'],
	 // set delimiter
	 delimiter: '-',
	 dateformat: function() {
	    // set dateformat
	    return 'yyyy' + this.delimiter + 'mm' + this.delimiter + 'dd'
	 }
      });
   }

   function initplayers() {
      // initialize #table head with no players
      $('#tableheader th').remove();
      $('#tableheader').append('<th><span id="hide"></span>' + applybutton + '</th>');
      $('#tableheader').append(plusplayertd);
   }

   function initrplayers() {
      $('#reserveheader th').remove();
      $('#reserveheader').append('<th class="noinput">Reserve</th>');
      $('#reserveheader').append(plusrplayerth);
   }

   function adaptsize(element) {
      let x = element.val();
      let h = $('#hide');
      h.text(x);
      let w = h.width();
      element.width(w);
   }

   window.handle_update_dates = function(){
      update_dates();
      table2data();
   }

   function update_dates(){
      let d = $('#firstdate').val();
      let dd = new Date(d);
      $('#weekday').html('&nbsp;&nbsp;'+weekdays[dd.getDay()]);
      $('#table>tbody>tr>td:first-child').each(function() {
	 $(this).text(d);
	 d = nextdate(d);
      });
      absent2table();
      initpicker();
   }

   function adddate() {
      var lastrow = $('#table>tbody>tr:last');
      $('#table>tbody>tr:last').after('<tr><td>hoppa</td></tr>');
      update_dates();
   }

   function deldate() {
      var lastrow = $('#table>tbody>tr:last');
      if (lastrow.attr('id') != 'firstrow') {
	 $('#table>tbody>tr:last').remove();
      }
      update_dates();
      filltable();
      table2data();
   }


   function checkplayers(){
      // remove double players
      players = [];
      $('#table .hplayer').each(function(){
	 players.push($(this).val());
      });
      //console.log("checkplayers",players);
      pset = new Set();
      initplayers();
      for (i in players) {
	 p = players[i];
	 pp = p.toLowerCase();
	 if (! pset.has(pp)) {
	    pset.add(pp);
	    addplayer(p,0);
	 }
      }
   }

   function checkrplayers(){
      // remove double rplayers
      players = [];
      $('#reserve .rplayer').each(function(){
	 players.push($(this).val());
      });
      //console.log("checkrplayers",players);
      pset = new Set();
      initrplayers();
      for (i in players) {
	 p = players[i];
	 pp = p.toLowerCase();
	 if (! pset.has(pp)) {
	    pset.add(pp);
	    addrplayer(p,0);
	 }
      }
   }

   function addplayer(p,dofill) {
      var x = '<input class="hplayer" type="text" value="'+p+'">';
      x += '<button class="delplayer">&nbsp;-&nbsp;</button>'
      $('.plusplayertd').remove();
      $('#table>thead>tr').append('<th>'+x+'</th>' + plusplayertd);
      addperson(p);
      if (dofill){
	 filltable();
	 table2data();
      }
      $('.hplayer').each(function() {
	 adaptsize($(this));
      });
   };

   function addrplayer(p,dofill) {
      var x = '<input class="rplayer" type="text" value="'+p+'">';
      x += '<button class="delrplayer">&nbsp;-&nbsp;</button>'
      $('.plusrplayerth').remove();
      $('#reserve>thead>tr').append('<th>'+x+'</th>' + plusrplayerth);
      addperson(p);
      if(dofill){
	 filltable();
	 table2data();
      }
      $('.rplayer').each(function() {
	 adaptsize($(this));
      });
   }

   function filltable() {
      //console.log("filltable");
      checkplayers();
      checkrplayers();
      let tprev = $('#table>tbody>tr:eq(0) td:eq(0)').text().substr(0,7);
      let classes = ['white','month'];
      let color = 0;
      // create array containing players:
      let players = table2players();
      //console.log("players:",players);

      let ns = players.length; // number of players

      $('#table>tbody>tr').each(function() {  // loop over dates
	 var t = $(this).children('td:eq(0)').text();
	 let mc = '';
	 if (tprev != t.substr(0,7)){
	    color = 1-color;  //switch color at month change
	    mc = ' monthchange';
	 }
	 $(this).children('td').remove();  // clear row
	 let row = "";
	 row += '<td class="rowdate '+classes[color]+mc+'">'+t+'</td>';
	 for (let i=0; i<ns; i++) {
	    row += '<td class="tablecell '+classes[color]+mc+'"> </td>';
	 }
	 row += '<td class="warning noinput"></td>';
	 $(this).append(row);
	 tprev = t.substr(0,7);
      });

      $('#sumplayrow td.sumplay').remove();
      $('#sumpayrow td.sumpay').remove();
      for (let i=0; i<ns; i++) {
	 $('#sumplayrow').append('<td class="sumplay sums noinput"></td>');
	 $('#sumpayrow').append('<td class="sumpay noinput"></td>');
      }

      let rplayers = table2rplayers();
      let s = new Set(players.concat(rplayers));
      let ppp = Array.from(s).sort(function(a,b){
	 let x = a.toLowerCase();
	 let y = b.toLowerCase();
	 if (x > y) {
	    return 1;
	 } else if (x<y) {
	    return -1;
	 }
	 return 0;
      });
      $('#emailbody>tr').remove();
      addemails(players);
      if (rplayers && rplayers.length >0 ) {
	 $('#emailbody').append('<tr><th class="reserveheader noinput" style="text-align:center;padding-left:0.5em;padding-right:0.5em;">'+
	    'Reserve</th><th></th><th></th><th></th></tr>');
      }
      addemails(rplayers);

      let varia = data.varia;
      if (!varia){ varia = "";}
      $('#varia').val(varia);
      autosize.update($('#varia'));

      absent2table();
      updatetable();

      let firstdate = table2firstdate();
      let wd = table2weekday();
      let lastdate = table2lastdate();
      $('#title').text('Tennisschema ' + wd + ' ' + firstdate + ' t/m ' + lastdate);
      document.title = makedate();
   }

   function addemails(p) {
      let ppp = p.slice();
      ppp.sort(function(a,b){
	 let x = a.toLowerCase();
	 let y = b.toLowerCase();
	 if (x > y) {
	    return 1;
	 } else if (x<y) {
	    return -1;
	 }
	 return 0;
      });
      for (let i in ppp){
	 let p = ppp[i];
	 let pp = p.toLowerCase();
	 let b = '<tr>';
	 b += '<td class="noinput">'+p+'</td>';
	 let name  = data.persons[pp].name;
	 let email = data.persons[pp].email;
	 let phone = data.persons[pp].phone;
	 if (!name)  { name = ""; }
	 if (!email) { email = ""; }
	 if (!phone) { phone = ""; }
	 b += '<td><input class="name"  type="text" value="'+name+ '"></td>';
	 b += '<td><input class="email" type="text" value="'+email+'"></td>';
	 b += '<td><input class="phone" type="text" value="'+phone+'"></td>';
	 b += '</tr>';

	 $('#emailbody').append(b);
      }
   }

   function sum(a){
      let s = 0;
      let n = a.length;
      for (let i=0; i<n; i++){
	 s += a[i];
      }
      return s;
   }

   function updatetable() {
      // create array containing players:
      var players = table2players();
      if (!players || players.length == 0){
	 return;
      }
      //console.log("players:",players);
      let numplayers = $('#dayplayers').val();
      // create set containing all dates:
      let dates = new Set();
      $('#table>tbody>tr').each(function () {
	 let d = $(this).children('td:eq(0)').text();
	 dates.add(d);
      });
      //console.log("dates:",dates);
      // create array containing players:
      var players = table2players();
      //console.log("players:",players);

      let ns = players.length; // number of players
      // count for each player the absent days
      let absents = [];
      for (let i=0; i<ns; i++){
	 absents[i] = 0;
      }
      $('#table>tbody>tr').each(function() {  // loop over dates
	 let kids = $(this).children();
	 let t = kids[0].textContent;
	 //console.log("ttttttt",t,kids[0].classList);
	 for (let j=0; j<ns; j++){
	    let i = j+1;
	    let x = kids[i].classList;
	    //console.log("classes:",x);
	    if (x.contains('absent') || x.contains('offday')){
	       absents[j] += 1;
	    }
	 }
      });
      //console.log("absents:",absents);
      /*
      //Tried something like this to improve result when there are many absent days,
      //but the method with a stepsize through the datums is better
      for (let i=0; i<ns; i++){
	 absents[i] = 0.5*absents[i];
      }
      */

      let nd         = $('#table>tbody>tr').length;  // number of dates in table
      //console.log("nd",nd);
      let startrow = 0;
      let rowstep = Math.ceil(nd/(10-$('#fiddle').val()));
      let count = 0;
      /*
      while (count < nd)
      {
	 let row = startrow;
	 while (row<nd){
	    $('#table>tbody>tr').eq(row).each(function() {
	       console.log("row",row,rowstep,count,$(this));
	       let kids = $(this).children();
	       console.log("row kid0:",kids[0],kids[0].textContent);
	    });
	    row += rowstep;
	    count += 1;
	 }
	 startrow += 1;
      }
      */

      // fill in table with codes for 'play', cannot play', 'free'
      startrow = 0;
      count = 0;
      while (count < nd) {
	 let row = startrow;
	 while (row<nd){
	    $('#table>tbody>tr').eq(row).each(function() {
	       let kids = $(this).children();
	       if (kids[0].classList.contains('offday')){
		  for (let i=1; i<ns+1; i++){ 
		     let x = 'td:eq('+i+')';
		     let y = $(this).children(x);
		     y.text("");
		  }
		  $(this).children('td:eq('+(ns+1)+')').text('ooo');
		  return;
	       }
	       let t = kids[0].textContent;  // the date of this row
	       actplayers = [];  // the actual players
	       for (let i=0; i<ns; i++){
		  actplayers[i] = 1;
	       }
	       // 'cannot' will contain the numbers of players that cannot play
	       cannot = [];
	       for (let i=0; i<ns; i++) {
		  let p = players[i];
		  let pp = p.toLowerCase();
		  if (data.persons[pp].absent[t]) {
		     actplayers[i] = 0;
		     cannot.push(i);
		  }
	       }

	       // remove players until desired number is obtained
	       while(sum(actplayers) > numplayers) {
		  // find the one with the least number of absents, this one will be free
		  let m = 1000000
		  let j=0
		  for (let i=0; i<ns; i++){
		     if (actplayers[i] == 1){
			if (absents[i] < m){
			   m = absents[i];
			   j = i;
			}
		     }
		  }
		  absents[j] += 1;
		  actplayers[j] = 0;
	       }
	       //console.log("actplayers:",t,actplayers,cannot,absents)

	       //console.log("actplayers:",actplayers);
	       // fill in row with results
	       for (let j=1; j<ns+1; j++){
		  let i = j-1;
		  let x = 'td:eq('+j+')';
		  let y = $(this).children(x);
		  if (actplayers[i] == 1){
		     y.text('X');
		  } else if (kids[j].classList.contains('absent')) {
		     y.text('-');
		  } else {
		     y.text('o');
		  }
	       }

	       let y = $(this).children('.warning');
	       if (sum(actplayers) < numplayers) {
		  //console.log("warning:",sum(actplayers),numplayers);
		  y.html('&nbsp;***');
	       } else {
		  y.text('');
	       }
	    });
	    row += rowstep;
	    count += 1;
	 }
	 startrow += 1;
      }
      
      //  determine who will pay the drinks

      let played  = [];
      let drink   = [];
      for (let i=0; i<ns; i++){
	 drink[i]   = 0;
	 played[i]  = 0;
      }
      $('#table>tbody>tr').each(function() {
	 let actplayers=[];
	 let kids = $(this).children();
	 // determine who is playing
	 for (let i=1; i<ns+1; i++) {
	    let x = 'td:eq('+i+')';
	    let y = $(this).children(x);
	    let text = y.text();
	    //console.log("iy",i,text);
	    if (text == 'X') {
	       actplayers[i-1] = 1;
	    }
	    else {
	       actplayers[i-1] = 0
	    }
	 }

	 for(let i=0; i<ns; i++){
	    played[i] += actplayers[i];
	 }

	 let dg = [];
	 for (let i=0; i<ns; i++){
	    if (actplayers[i] == 1){
	       let d1 = drink[i] / played[i];
	       dg.push([d1,i]);
	    }
	 }
	 //console.log("actplayers",actplayers);
	 dg.sort();
	 //console.log("dg:",dg);

	 let payers = new Set();
	 let numpayers = $('#daypayers').val();
	 if (numpayers > numplayers) {
	    numpayers = numplayers;
	    $('#daypayers').val(numpayers);
	 }

	 let n = numpayers;
	 if (n > sum(actplayers)){
	    n = sum(actplayers);
	 }

	 for (let i=0; i<n; i++) {
	    payers.add(dg[i][1]);
	 }
	 payers.forEach(function(x){
	    drink[x] += 1;
	 });

	 for (let j=1; j<ns+1; j++){
	    let i = j-1;
	    let x = 'td:eq('+j+')';
	    let y = $(this).children(x);
	    if (actplayers[i] == 1){
	       if (payers.has(i)) {
		  y.text('Xd');
	       }
	    }
	 }

	 // create totals of paymants
	 //console.log("payers",payers);
	 let sumplayrow = $('#sumplayrow');
	 let sumpayrow = $('#sumpayrow');
	 for (let i=1; i<ns+1; i++) {
	    let x = 'td:eq('+i+')';
	    let y = sumplayrow.children(x);
	    y.text(played[i-1]);
	    y = sumpayrow.children(x);
	    y.text(drink[i-1]);
	 }

	 if (numpayers > 0) {
	    $('#sumpayrow').show();
	 }
	 else {
	    $('#sumpayrow').hide();
	 }
      });
   }

   function needupdate(x) {
      if (x) {
	 $('#fill').css("color","black").css("background","yellow").attr("need","yes");
	 return;
      }
      $('#fill').css("color","gray").css("background","white").attr("need","no");
   }

   function addperson (p)
   {
      // adds person p if necessary
      pp = p.toLowerCase();
      if (!data.persons[pp]){
	 data.persons[pp] = {};
	 data.persons[pp]["email"]  = "";
	 data.persons[pp]["phone"]  = "";
	 data.persons[pp]["absent"] = {};
      }
   }

   function table2players() {
      // return players found in #table
      let players = [];
      $('#table>thead>tr>th>input').each(function() {
	 var p = $(this).val();
	 players.push(p);
      });
      return players;
   }

   function table2rplayers() {
      // return players found in #table
      let players = [];
      $('#reserve>thead>tr>th>input').each(function() {
	 var p = $(this).val();
	 players.push(p);
      });
      return players;
   }

   function table2data() {
      // extract data from #table, put this in 'data'
      let ps = table2players();
      data.players = ps.slice();
      ps = table2rplayers();
      data.rplayers = ps.slice();

      $('#table>tbody>tr').each(function() {
	 var kids = $(this).children();
	 //console.log("kids",kids[1]);
	 var d = kids[0].textContent;
	 if (kids[0].classList.contains('offday')) {
	    data.offdays[d] = 1;
	 } else {
	    delete(data.offdays[d]);
	 }
	 //d = $(this).children(':first').text();
	 //console.log("table2data",d);
	 for (j in data.players) {
	    i = parseInt(j);
	    var p = data.players[i];
	    var pp = p.toLowerCase();
	    //console.log("absent:",i,p,data.persons[pp].absent);
	    var a = kids[1+i];
	    //console.log("a",1+i,a,a.classList);
	    if (a.classList.contains('absent')) {
	       data.persons[pp].absent[d]=1;
	    } else {
	       delete(data.persons[pp].absent[d]);
	    }
	 }

      });
      //console.log(data.players);
      data["programid"]   = programid;
      data.startdate      = $('#firstdate').val();
      data.ndates         = $('#table>tbody>tr').length;
      data.dayplayers     = $('#dayplayers').val();
      data.daypayers      = $('#daypayers').val();
      data.fiddle         = $('#fiddle').val();

      data.version.date   = versiondate;
      data.version.rev    = versionrev;

      $('#emailbody>tr').each(function() {
	 let kids = $(this).children();
	 if (!kids[0].classList.contains('reserveheader')){
	    let p = kids[0].textContent;
	    let pp = p.toLowerCase();

	    let name = kids[1].children[0].value;
	    //console.log("name:",name);
	    data.persons[pp].name = name;

	    let email = kids[2].children[0].value;
	    //console.log("email:",email);
	    data.persons[pp].email = email;

	    let phone = kids[3].children[0].value;
	    //console.log('phone',phone);
	    data.persons[pp].phone = phone;
	 }
      });

      data.varia = $('#varia').val();

   }

   function absent2table() {
      // gets absent's from data, puts this in #table
      players = table2players();
      $('#table>tbody>tr').each(function(){
	 var kids = $(this).children();
	 var d = kids[0].textContent;
	 if (data.offdays[d]) {
	    $(this).children().addClass('offday');
	 }
	 for (j in players){
	    var i = parseInt(j);
	    p = players[i];
	    pp = p.toLowerCase();
	    if (typeof data.persons[pp].absent === "undefined"){
	       data.persons[pp].absent = {};
	    }
	    if (data.persons[pp].absent[d]) {
	       //console.log("absent:",p,pp,d,i+1);
	       var x = 'td:eq('+(i+1)+')';
	       $(this).children(x).addClass('absent');
	    }
	 }
      });
   }


   function data2table() {
      // extract data from data, puts it in #table and some others
      $('#top').html(inittop());
      $('#firstdate').val(data.startdate);
      $('#dayplayers').val(data.dayplayers);
      $('#daypayers').val(data.daypayers);
      if (typeof data.fiddle === "undefined"){
	 data.fiddle = 1;
      }
      $('#fiddle').val(data.fiddle);
      $('#table').remove();
      $('#tstable').html( inittable());

      try {
	 oldversiondate = data.version.date;
	 oldversionrev  = data.version.rev;
      }  catch(e) {
	 //console.log("data2table catch");
	 oldversiondate = date2iso(new Date());
	 oldversionrev  = 1;
	 data["version"] = {};
	 data.version["date"] = oldversiondate;
	 data.version["rev"]  = oldversionrev;
      }

      versiondate = oldversiondate;
      versionrev  = oldversionrev;

      $('#versiondate').text(versiondate);
      $('#versionrev').text(versionrev);

      initrplayers();
      autosize($('#varia'));
      for (let i in data.players) {
	 addplayer(data.players[i]);
      }

      for (let i in data.rplayers) {
	 addrplayer(data.rplayers[i]);
      }

      update_dates();
      for (let i=1; i<data.ndates; i++){
	 //console.log("data2table: adddate",i);
	 adddate();
      }
   }

   function table2firstdate() {
      return $('#table .rowdate:first').text();
   }

   function table2lastdate() {
      return $('#table .rowdate:last').text();
   }

   function table2weekday() {
      let dd = new Date(table2firstdate());
      return weekdays[dd.getDay()];
   }

   function data2crc()
   {
      // return stringified data, version replaced by constant value
      let d = data.version.date;
      let r = data.version.rev;
      data.version.date = "2000-01-01";
      data.version.rev = 1;
      let s = JSON.stringify(data);
      data.version.date = d;
      data.version.rev = r;
      return s;
   }


   function makedate() {
      let firstdate = table2firstdate();
      let wd = table2weekday();
      let lastdate = table2lastdate();
      r = firstdate+'-'+lastdate+'-'+wd;
      return r;
   }

   function download() { // from here to user's file system  (save)
      checkcrc();
      table2data();
      let myBlob = new Blob([JSON.stringify(data,null,1)]);
      let blobUrl = URL.createObjectURL(myBlob);
      let link = document.createElement("a"); // Or maybe get it from the current document
      link.href = blobUrl;

      link.download  = 'sessie-'+makedate()+'.txt';
      document.title = 'sessie-'+makedate();
      link.innerHTML = "Download";
      link.hidden = 1;
      link.click();
      crc = data2crc();
   }

   function upload() {   // from user's file system to here  (open)
      //console.log("this is upload");
      var link = document.createElement("input");
      link.id = 'uploadinput';
      link.name = 'files[]';
      link.type = 'file';
      $('#menu').append(link);
      link.click();
      link.style = 'visibility: hidden;';
   }

   function print() {
      //console.log("this is print");
      const title = 'tennis-'+makedate();
      checkcrc();
      document.title = title;

      $('#tstable').print({noPrintSelector:'.delplayer',title:title});
   }

   function checkcrc() {  // used by save and print
      // if there is some change, adapt version
      table2data();
      if (crc != data2crc()) {
	 //console.log("checkcrc:");
	 let today = date2iso(new Date());
	 if (oldversiondate == today){
	    if (oldversionrev == versionrev){
	       versionrev++;
	    }
	 } else {
	    versiondate = today;
	    versionrev = 1;
	 }
      }
      $('#versiondate').text(versiondate);
      $('#versionrev').text(versionrev);
   }

   function jsonerror() {
      alert('Het gekozen bestand is niet bruikbaar.\n'+
	 'Gebruik alleen bestanden die zijn gemaakt met dit programma.');
   }

   $(document).on('change', '#firstdate',function() {
      update_dates();
   });

   $(document).on('change','#uploadinput',function(evt) {
      if(!window.FileReader) 
      {
	 //console.log("nono");
	 return; // Browser is not compatible
      }

      filename = evt.target.files[0].name;
      //console.log("input...",filename);

      var reader = new FileReader();

      reader.onload = function(evt) {
	 if(evt.target.readyState != 2) return;
	 if(evt.target.error) {
	    alert('Error while reading file');
	    return;
	 }

	 //console.log("input2...");
	 filecontent = evt.target.result;
	 //console.log("filecontent:",filecontent);
	 try {
	    data = JSON.parse(filecontent);
	 } catch(e) {
	    jsonerror();
	    return;
	 }
	 data2table();
	 //console.log("programid__",programid,data.programid);
	 if (!data.programid || data.programid != programid){
	    jsonerror();
	    return;
	 }
	 crc = data2crc();
	 filltable();

	 $('#text').val(filecontent);

      };

      reader.readAsText(evt.target.files[0]);
   });

   $(document).on('change click','.name,.email,.phone,#varia',function() {
      $('.apply').removeClass('needapply');
      table2data();
      //console.log("change name/email/phone",crc,data2crc());
   });

   // kind of main

   let versiondate, versionrev;
   let oldversiondate, oldversionrev;

   let crc;

   $(window).on('beforeunload', function () {
      if (crc != data2crc()) {
	 return 'You haven\'t saved your changes.';
      }
   });


   let plusbutton     = '<button class="adddate">&nbsp;+&nbsp;</button>';
   let minbutton      = '<button class="deldate">&nbsp;-&nbsp;</button>';
   let plusplayer     = '<button class="addplayer">&nbsp;+&nbsp;</button>';
   let plusrplayer    = '<button class="addrplayer">&nbsp;+&nbsp;</button>';
   let minplayer      = '<button class="delplayer">&nbsp;-&nbsp;</button>';
   let minrplayer     = '<button class="delrplayer">&nbsp;-&nbsp;</button>';
   let downloadbutton = '<button id="download">&nbsp;Bewaar deze sessie&nbsp;</button>';
   let uploadbutton   = '<button id="upload">&nbsp;Open bewaarde sessie&nbsp;</button>';
   let printbutton    = '<button id="print">&nbsp;print/pdf&nbsp;</button>';
   let helpbutton     = '<button id="helpbutton">&nbsp;help&nbsp;</button>';
   // let onhelpbutton   = '<button id="onhelpbutton">&nbsp;terug naar schema&nbsp;</button>';
   let fillbutton     = '<button id="fill">Ververs</button>';
   let playersbutton  = '<input id="dayplayers" type="number" min="1" max="20" size="4"/>';
   let payersbutton   = '<input id="daypayers" type="number" min="0" max="20" size="4"/>';
   let fiddlebutton   = '<input id="fiddle" type="number" min="1" max="9" size="4"/>';
   let applybutton    = '<button class="apply">Verwerk</button>';

   let plusplayertd = '<td class="plusplayertd">&nbsp;'+plusplayer+'</td>';
   let plusrplayerth = '<th class="plusrplayerth">&nbsp;'+plusrplayer+'</th>';

   $('#tstable').html(inittable());
   initrplayers();
   $('#help').html(inithelp());
   $('#top').html(inittop());
   $('#menu').html(initts());

   //console.log("firstcell",$('#tableheader th:eq(0)').text());
   initpicker();
   //console.log("firstcell",$('#tableheader th:eq(0)').text());

   initplayers();

   data = initdata();
   if (data.programid != programid) {
      $("#menu").hide();
      $("#tstable").hide();
      $("#help").hide();
      alert('Interne fout: ongeldige data\n'+
	 'Verwacht program id "'+programid+'" maar kreeg "'+data.programid+'"');
      return false;
   }

   data2table();
   filltable();

   table2data();
   crc = data2crc();
   //console.log("crc",crc);

   update_dates();

   needupdate(0);

   $('#help').hide();
   $('#menu').show();
   $('#tstable').show();


   $(".adddate").click(function() {
      adddate();
      filltable()
      table2data();
   });

   $(".deldate").click(function() {
      deldate();
      table2data();
   });


   $(document).on('click', '.addplayer',function() {
      //console.log("addplayer");
      var ns = $('#table>thead>tr th').length;
      var sp = '';
      addplayer(sp,1);
      filltable();
      table2data();
      $('input.hplayer[value=""]').focus();
   });

   $(document).on('click', '.addrplayer',function() {
      //console.log("addrplayer");
      addrplayer('',1);
      checkrplayers();
   });

   $(document).on('input', '.hplayer,.rplayer',function() {
      adaptsize($(this));
   });

   $(document).on('click','.delplayer',function() {
      $(this).closest('th').remove();
      filltable();
      $('.plusplayertd').remove();
      $('#table>thead>tr').append(plusplayertd);
      table2data();
   });

   $(document).on('click','.delrplayer',function() {
      $(this).closest('th').remove();
      filltable();
      $('.plusrplayerth').remove();
      $('#reserveheader').append(plusrplayerth);
      table2data();
   });

   $(document).on('click','#download',function() {
      //console.log("clicked download");
      download();
   });

   $(document).on('click','#upload',function() {
      //console.log("clicked upload");
      upload();
   });

   $(document).on('click','#print',function() {
      //console.log("clicked print");
      print();
   });

   $(document).on('click','#helpbutton',function() {
      //console.log("clicked help");
      $('#tstable').hide();
      $('#menu').hide();
      $('#help').show();
   });

   $(document).on('click','.onhelpbutton',function() {
      //console.log("clicked nohelp");
      $('#menu').show();
      $('#tstable').show();
      $('#help').hide();
   });


   $(document).on('click','#table .tablecell',function() {
      var e = $(this).parent().children(':first');
      var t = e.text();
      //console.log("tablecell",t);
      let x = $(this);
      $(this).toggleClass("absent");
      table2data();
      updatetable();
   });

   $(document).on('click','#table .rowdate',function() {
      var d = $(this).text();
      //console.log("click rowdate",d);
      $(this).parent().children('.tablecell,.rowdate').toggleClass('offday');
      table2data();
      updatetable();
   });

   $(document).on('click','#fill',function() {
      if ($(this).attr("need") == "yes"){
	 filltable();
	 updatetable();
	 needupdate(0);
      }
   });

   $(document).on('click','.apply',function() {
      filltable();
      table2data();
      $('.apply').removeClass('needapply');
   });

   $(document).on('keydown','#dayplayers,#daypayers,#firstdate,#fiddle',function() {
      return false;
   });

   $(document).on('keydown','.hplayer, .rplayer, #emails input, #tvaria',function() {
      //console.log("keydown..",$(this).val());
      $('.apply').addClass('needapply');
   });

   $(document).on('change','.hplayer,.rplayer',function() {
      //console.log("change..",$(this).val());
      filltable();
      table2data();
   });

   $(document).on('change','#dayplayers',function() {
      let v = $(this).val();
      //console.log("dayplayers:",v);
      if (v && v>0) {
	 filltable();
	 table2data();
      }
   });

   $(document).on('change','#daypayers',function() {
      let v = $(this).val();
      //console.log("daypayers:",v);
      if (v && v>=0) {
	 filltable();
	 table2data();
      }
   });

   $(document).on('change','#fiddle',function() {
      let v = $(this).val();
      //console.log("fiddle:",v);
      if (v && v>=0) {
	 filltable();
	 table2data();
      }
   });

   function inittable(){
      let t = '';
      t += '<br><div id="titlediv" style="width:60em; margin:auto; text-align: center;cursor:default;">';
      t +=    '<span id="title" class="noinput"></span>';
      t +=    '&nbsp&nbsp;versie&nbsp;<span id="versiondate"></span>.<span id=versionrev></span>';
      t +=    '<span id="quickhelp" class="noprint"><br>Aan- of uitzetten verhindering speler of datum: klik in schema.</span>';
      t += '</div>';
      t += '<table id="table">';
      t +=    '<thead>';
      t +=       '<tr id="tableheader">';
      t +=           '<th>xxx</th>';
      t +=       '</tr>';
      t +=    '</thead>';
      t +=    '<tbody id="tablebody">';
      t +=        '<tr id="firstrow"><td>startdatum </td></tr>';
      t +=    '</tbody>';
      t +=    '<tfoot id="tablefooter">';
      t +=        '<tr id="sumplayrow"><td class="sums noinput">Te spelen:</td></tr>';
      t +=        '<tr id="sumpayrow"><td class="noinput">Drankjes:</td></tr>';
      t +=     '</tfoot>';
      t += '</table>';

      t += '<br>';
      t += '<table id="legenda" class="noinput">';
      t +=    '<tbody>';
      t +=        '<tr>';
      t +=            '<td>Legenda:</td>';
      t +=            '<td style="text-align:right;">&nbsp; X:&nbsp;</td><td>&nbsp;speelt</td>';
      t +=            '<td                          >&nbsp; d:&nbsp;</td><td>&nbsp;betaalt drankje</td>';
      t +=        '</tr>';
      t +=        '<tr>';
      t +=            '<td></td>';
      t += 	      '<td style="text-align:right;">&nbsp; -:&nbsp;</td><td>&nbsp;is verhinderd</td>';
      t +=            '<td                          >&nbsp; o:&nbsp;</td><td>&nbsp;heeft vrij</td>';
      t +=        '</tr>';
      t +=        '<tr>';
      t +=            '<td></td>';
      t +=            '<td style="text-align:right;">&nbsp; ***:&nbsp;</td><td>&nbsp;te weinig beschikbare spelers</td>';
      t +=            '<td style="text-align:right;">&nbsp; ooo:&nbsp;</td><td>&nbsp;geen speeldag</td>';
      t +=        '</tr>';
      t +=    '</tbody>';
      t += '</table>';

      t += '<br>';
      t += '<table id="reserve">';
      t +=    '<thead>';
      t +=        '<tr id="reserveheader">';
      t +=           '<th>reserve</th>'
      t +=           plusrplayerth;
      t +=        '</tr>';
      t +=    '</thead>';
      t +=    '<tbody style="display:none;">';
      t +=        '<tr><td> </td></tr>';
      t +=    '</tbody>';
      t += '</table>';

      t += '<br>';
      t += '<table id="emails">';
      t +=    '<thead id="emailheader" class="noinput">';
      t +=        '<tr>';
      t +=           '<th>&nbsp;Speler&nbsp;</th>';
      t +=           '<th align="center">&nbsp;Naam&nbsp;</th>';
      t +=           '<th align="center">&nbsp;Email&nbsp;</th>';
      t +=           '<th align="center">&nbsp;Telefoon&nbsp;</th>';
      t +=         '</tr>';
      t +=     '</thead>';
      t +=     '<tbody id="emailbody">';
      t +=     '</tbody>';
      t +=     '<tfoot id="emailfooter">';
      t +=     '</tfoot>';
      t += '</table>';

      t += '<br>';
      t += '<table id="tvaria">';
      t += '<tr><td>';
      t +=     '<textarea id="varia" placeholder="Vul hier reserve spelers, tijden en plaatsen in.">';
      t +=     '</textarea>';
      t += '</td></tr>';
      t += '</table>';
      t += '<div class="noprint"><br><br><br><br><br><br><br><br><br><br></div>';

      return t;
   }

   function inithelp() {
      return ` 
      <div id="helptext">
	 <button class="onhelpbutton">&nbsp;terug naar schema&nbsp;</button>
	 <br>
	 <h2>Handleiding</h2>
	 <H3>Beschrijving van het programma</H3>
	 <p>
	 Met dit programma stelt u een wekelijks tennisschema samen. U stelt in welke 
	 spelers meedoen, hoeveel spelers er op een dag spelen en wie het drankje betaalt. 
	 U stelt verhinderdata per persoon in en algehele verhinderdata (Kerstmis, bijvoorbeeld). 
	 Daarnaast voert u de e-mail adressen, telefoonnummers en volledige namen van de spelers en reservespelers in. 
	 Er is ook ruimte voor vrije opmerkingen: locatie en dergelijke. 
	 </p>
	 <p>
	 Het programma stelt zelf het schema samen. Er wordt <b>automatisch</b> voor gezorgd dat 
	 alle spelers een zoveel als mogelijk gelijk aantal speel- en koffiebeurten hebben.
	 </p>
	 <p>
	 <b>De gegevens kunnen worden opgeslagen op uw eigen PC en later weer worden opgehaald.</b> 
	 <br>Het samengestelde schema kan worden ge&euml;exporteerd naar een pdf bestand dat u naar de deelnemende spelers kunt e-mailen.
	 </p>
	 <h3>Functionaliteit</h3>
	 <ul>
	    <li> <span class="firstli">Algemeen: starten, verwerken en klikken</span>
	       <ul>
		  <li>Het programma start met een fantasie-schema, geschikt om te oefenen <b>of</b> 
		     vanuit hier uw eigen schema samen te stellen met uw eigen spelers.</li>
		  <li>Linksboven in de tabel ziet u een button met de tekst <b>Verwerk</b>. Deze kleurt 
		     oranje als ingevoerde gegevens in de tabel nog niet zijn verwerkt. 
		     Klik op deze button om de gegevens te verwerken.</li>
		  <li> Alle velden in het schema zijn aanklikbaar om verhinderdagen/-data aan te geven.<br>
	       </ul>   
		  </li>
		  <li><span class="firstli">Begindatum instellen</span>
		     <ul>
			<li>Klik op het veld achter <b>Startdatum:</b> en kies de gewenste startdatum met 
			   de kalender-pop-up. </li>
			<li>Eindig de kalender-pop-up door op <b>X</b> te klikken.</li>
		     </ul>
		  </li>
		  <li><span class="firstli">Aantal spelers per dag instellen</span>
			<br>Stel het aantal spelers per dag in met de button achter <b>Spelers/dag</b>. 
		  </li>
		  <li><span class="firstli">Aantal drankjes per dag instellen</span>
			<br>Stel het aantal drankjes per dag in met de button achter <b>Drankjes/dag</b>. 
		  </li>
		  <li><span class="firstli">Niet tevreden met resulaat?</span>
			<br>Door het getal achter <b>Fiddle</b> te veranderen, verandert het resultaat,
			    met name als er veel verhinderdagen zijn.
		  </li>

		  <li><span class="firstli">Toevoegen speler</span>
		     <br>Klik op de <span class="fat"><span class="fatbig">+</span> button</span> achter de namen van de spelers 
		     en vul een naam in in de eerste rij van de nieuwe kolom. 
		  </li>
		  <li><span class="firstli">Verwijderen speler</span>
		     <br>Klik op de <span class="fat"><span class="fatbig">-</span> button</span> achter de naam van een speler. 
		  </li>
		  <li><span class="firstli">Toevoegen reservespeler</span>
		     <br>In de tabel, gemarkeerd door <b>Reserve</b>, klik op de 
		     <span class="fat"><span class="fatbig">+</span> button</span> en vul de naam 
		     van de reservespeler in.
		  </li>
		  <li><span class="firstli">Verwijderen reservespeler</span> 
		     <br>Klik op de <span class="fat"><span class="fatbig">-</span> button</span> achter de te verwijderen reservespeler. 
		  </li>

		  <li><span class="firstli">Speeldata toevoegen</span>
		     <br>Klik op de <span class="fat"><span class="fatbig">+</span> button</span> achter <i>Speeldata</i> 
		  </li>
		  <li><span class="firstli">Speeldata verwijderen</span>
		     <br>Klik op de <span class="fat"><span class="fatbig">-</span> button</span> achter <i>Speeldata</i> 
		  </li>

		  <li><span class="firstli">Instellen/verwijderen verhinderdag van een speler</span>
			<ul>
			<li>Ga naar de rij met de betreffende datum en klik in de kolom onder de naam van de betreffende speler. 
			   Er verschijnt nu een - teken en de cel wordt roze.</li>
			   <li>Door nogmaals op de cel te klikken wordt de verhinderdag verwijderd.</li>
			 </ul>
		  </li>
		  <li><span class="firstli">Instellen/verwijderen datum waarop niet kan worden gespeeld (bv. Kerstmis)</span> 
		     <ul>
			<li>Klik links in de tabel op de gewenste datum: de rij kleurt geel.</li>
			<li>Door nogmaals op die datum klikken wordt de wijziging ongedaan gemaakt.</li>
		     </ul> 
		  </li>
		  <li><span class="firstli">Gegevens van spelers</span>
		     <br>Naam, e-mail en telefoon van de spelers vult u in in de tabel onder het schema. 
		  </li>
		  <li><span class="firstli">Extra informatie toevoegen</span> 
		     <ul>
			<li>Onder de tabel met de gegevens van de spelers is een veld waarin u 
			   willekeurige gegevens kunt intypen, zoals plaats, tijd, locatie.</li>
			<li>Klik hierna de <b>Verwerk-button boven in het schema.</b></li>
		     </ul>
		  </li>
		  <li><span class="firstli">Bewaren van de gegevens van het ingevulde schema</span> 
		     <br>Klik op de button <b>bewaar deze sessie</b> 
		     <br>De browser (Firefox, Edge, Chrome...) presenteert dan een scherm waarin u de 
		     naam van het bestand en directory kunt aangeven. 
		     Als suggestie wordt een naam gegeven met daarin datum en dergelijke van het schema. 
		  </li>
		  <li><span class="firstli">Ophalen van bewaarde gegevens</span>   
		     <br>Klik op de button <b>open bewaarde sessie</b>. 
		     <br>De browser (Firefox, Edge, Chrome...) presenteert dan een scherm waarin u 
		     het gewenste, voorheen bewaarde, kunt selecteren en openen. 
		  </li>
		  <li><span class="firstli">Bewaren als pdf of printen</span> 
		     <br>Klik op de button <b>print/pdf</b>. 
		     <br>De browser (Firefox, Edge, Chrome...) presenteert een scherm waarin u de 
		     keuze maakt tussen printen en bewaren als pdf. 
		     Tevens wordt een voorbeeld getoont van hoe een en ander er uit gaat zien. 
		     Dit voorbeeld moet u niet te letterlijk nemen, soms zijn bv. enkele horizontale lijnen
		     weggevallen. 
		     <br>Wel kunt u hier, indien nodig, de groote van de pagina aanpassen middels 
		     de opties "fit to page", "scale in %", of iets dergelijks. De preciese opties 
		     zijn afhankelijk van de gebruikte browser en/of printer. 
		     <br>Het pdf bestand kunt u als bijlage emailen naar de spelers. 
		  </li>
		  <li><span class="firstli">Handleiding</span> 
		     <br>Klik de button <b>help</b>. 
		     Keeer terug naar het schema door op de button <b>terug naar schema</b> te kilkken. 
		  </li>
	 </ul>
	 <button class="onhelpbutton">&nbsp;terug naar schema&nbsp;</button>
      </div>
  `
   }

   function inittop() {
      let t = '';
      t += '<div id="topdiv" style="width: 40em; margin:auto; text-align: center; cursor: default;">';
      t += '<span id="toptitle";style="font-size:larger;" class="noinput">Willem\'s tennis scheduler</span>';
      t += '</div>';
      t += '<br>';
      return t;
   }

   function initts() {
      let page = "";
      page += '<table id="menutable"><tr><td>';
      page += 'Startdatum: ';
      let now = new Date();
      let nowstr = date2iso(now);
      let fdate = nextdate(nowstr);
      page += '<input size="8" type="text" id="firstdate" value="'+fdate+'" autocomplete="off" style="fnt-size:larger;">'
      page += '&nbsp;Speeldag:'
      page += '<span id="weekday"></span>';
      page += '&nbsp;';
      page += '<br><br>';
      page += 'Speeldata: &nbsp;';
      page += plusbutton;
      page += minbutton;

      page += '&nbsp;Spelers/dag:&nbsp;';
      page += playersbutton;

      page += '&nbsp;Drankjes/dag:&nbsp;';
      page += payersbutton;
      page += '&nbsp;Fiddle:&nbsp;';
      page += fiddlebutton;
      page += '<br><br>';
      page += downloadbutton;
      page += '&nbsp;';
      page += uploadbutton;
      page += '&nbsp;';
      page += printbutton;
      page += '&nbsp;';
      page += helpbutton;
      page += '</td></tr></table>';

      return page;
   }

   function initdata() {
      return {
	 "programid": "Willem's tennis scheduler",
	 "version": {
	    "date": "2024-01-01",
	    "rev": 1
	 },
	 "persons": {
	    "beckenbauer": {
	       "absent": {
		  "2012-02-04": 1,
		  "2012-02-06": 1
	       },
	       "email": "hans@example.com",
	       "phone": "06-999900",
	       "name": "Hans B."
	    },
	    "piet": {
	       "email": "pietersma@casema.nl",
	       "phone": "06-12345678",
	       "absent": {
		  "2024-04-19": 1
	       },
	       "name": "Piet Pietersma"
	    },
	    "carla": {
	       "email": "carla@abc.com",
	       "phone": "060-123456789",
	       "absent": {
		  "2024-02-09": 1,
		  "2024-04-26": 1
	       },
	       "name": "C. Mariskadottir"
	    },
	    "mies": {
	       "email": "mies@example.com",
	       "phone": "070-333777666",
	       "absent": {
		  "2024-01-26": 1,
		  "2024-03-08": 1
	       },
	       "name": "Mies Muizenis"
	    },
	    "wim": {
	       "email": "wc@example.com",
	       "phone": "06-9876545678",
	       "absent": {
		  "2024-02-09": 1,
		  "2024-03-29": 1
	       },
	       "name": "Wim Carlsson"
	    },
	    "zus": {
	       "email": "zschaafsma2@trala.com",
	       "phone": "08-890",
	       "absent": {
		  "2024-03-15": 1
	       },
	       "name": "Z. Schaafsma"
	    },
	    "jet": {
	       "email": "kamervraag@gmail.com",
	       "phone": "geen telefoon",
	       "absent": {
		  "2024-02-09": 1
	       },
	       "name": "Jet Kamervraag"
	    },
	    "": {
	       "email": "",
	       "phone": "",
	       "absent": {},
	    },
	    "jongejan": {
	       "email": "piet@example.com",
	       "phone": "o70-piet",
	       "absent": {},
	       "name": "piet jongejan-staas"
	    },
	    "pietersma": {
	       "email": "jan@example.com",
	       "phone": "06-jan",
	       "absent": {},
	       "name": "jan pietersma "
	    },
	    "trijntje": {
	       "email": "oo@example.com",
	       "phone": "06-trijntje",
	       "absent": {},
	       "name": "T. Oosterhuis"
	    },
	    "pieter": {
	       "email": "kamgaren@example.com",
	       "phone": "0800-12345",
	       "absent": {},
	       "name": "Pieter Kamgaren"
	    }
	 },
	 "players": [
	    "Piet",
	    "Carla",
	    "Mies",
	    "Wim",
	    "Zus",
	    "Jet"
	 ],
	 "rplayers": [
	    "Trijntje",
	    "Pieter"
	 ],
	 "offdays": {
	    "2024-03-15": 1
	 },
	 "startdate": "2024-01-05",
	 "ndates": 21,
	 "dayplayers": "4",
	 "daypayers": "1",
	 "fiddle": "1",
	 "varia": "Locatie: OLTC Tijd: 10:00-11:00\n\nLet op: wie het rondje betaalt, wordt ook geacht de baan te reserveren."
      }
   }
})
